@@ -57,7 +57,7 @@ void y_log_message(const unsigned long type, const char * message, ...) {
   (void)(message);
 }
 
-int y_close_logs() {
+int y_close_logs(void) {
   return 1;
 }
 #endif
@@ -207,6 +207,7 @@ void * ulfius_uri_logger (void * cls, const char * uri) {
   UNUSED(cls);
 
   if (con_info != NULL) {
+    memset(con_info, 0, sizeof(struct connection_info_struct));
     con_info->callback_first_iteration = 1;
     con_info->u_instance = NULL;
     u_map_init(&con_info->map_url_initial);
@@ -217,7 +218,8 @@ void * ulfius_uri_logger (void * cls, const char * uri) {
       return NULL;
     }
 
-    if (NULL == con_info->request || ulfius_init_request(con_info->request) != U_OK) {
+    if (ulfius_init_request(con_info->request) != U_OK) {
+      y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error initializing con_info->request");
       ulfius_clean_request_full(con_info->request);
       o_free(con_info);
       return NULL;
@@ -316,50 +318,91 @@ static int mhd_iterate_post_data (void * coninfo_cls, enum MHD_ValueKind kind, c
                                   const char * transfer_encoding, const char * data, uint64_t off, size_t size) {
 #endif
   struct connection_info_struct * con_info = coninfo_cls;
-  size_t cur_size = size;
-  char * data_dup, * filename_param;
+  size_t data_size = size, cur_size;
+  char * filename_param = NULL, * data_concat = NULL;
+  const char * cur_data;
+#if MHD_VERSION >= 0x00097002
+  enum MHD_Result ret = MHD_YES;
+#else
+  int ret = MHD_YES;
+#endif
   UNUSED(kind);
 
-  if (filename != NULL && con_info->u_instance != NULL && con_info->u_instance->file_upload_callback != NULL) {
-    if (con_info->u_instance->file_upload_callback(con_info->request, key, filename, content_type, transfer_encoding, data, off, size, con_info->u_instance->file_upload_cls) == U_OK) {
-      return MHD_YES;
-    } else {
-      return MHD_NO;
+  if (con_info->u_instance == NULL) {
+    ret = MHD_NO;
+  } else if (filename != NULL && con_info->u_instance->file_upload_callback != NULL) {
+    if (con_info->u_instance->file_upload_callback(con_info->request, key, filename, content_type, transfer_encoding, data, off, size, con_info->u_instance->file_upload_cls) != U_OK) {
+      ret = MHD_NO;
     }
   } else {
-    if (con_info->u_instance) {
-      if (con_info->u_instance->check_utf8 && (utf8_check(key, o_strlen(key)) != NULL || data == NULL || utf8_check(data, o_strlen(data)) != NULL || (filename != NULL && utf8_check(filename, o_strlen(filename)) != NULL))) {
-        return MHD_YES;
+    
+    do {
+      if (con_info->u_instance->check_utf8) {
+        if (utf8_check(key, o_strlen(key)) != NULL || data == NULL || utf8_check(data, o_strlen(data)) != NULL || (filename != NULL && utf8_check(filename, o_strlen(filename)) != NULL)) {
+          break;
+        }
+      }
+      
+      if (con_info->max_post_param_size && off > con_info->max_post_param_size) {
+        break;
+      }
+      
+      if (off + size > con_info->max_post_param_size) {
+        data_size = con_info->max_post_param_size - off;
+      }
+      
+      if (filename != NULL) {
+        filename_param = msprintf("%s_filename", key);
+        if (!u_map_has_key((struct _u_map *)con_info->request->map_post_body, filename_param) && u_map_put((struct _u_map *)con_info->request->map_post_body, filename_param, filename) != U_OK) {
+          y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error u_map_put filename value");
+        }
+        cur_data = u_map_get((struct _u_map *)con_info->request->map_post_body, key);
+        cur_size = u_map_get_length((struct _u_map *)con_info->request->map_post_body, key);
+        if (cur_data != NULL) {
+          if (off) {
+            if (u_map_put_binary((struct _u_map *)con_info->request->map_post_body, key, data, cur_size, data_size) != U_OK) {
+              ret = MHD_NO;
+              break;
+            }
+          } else {
+            if (u_map_put_binary((struct _u_map *)con_info->request->map_post_body, key, ",", cur_size, 1) != U_OK ||
+                u_map_put_binary((struct _u_map *)con_info->request->map_post_body, key, data, cur_size+1, 1) != U_OK) {
+              ret = MHD_NO;
+              break;
+            }
+          }
+        } else {
+          if (u_map_put_binary((struct _u_map *)con_info->request->map_post_body, key, data, 0, data_size) != U_OK) {
+            ret = MHD_NO;
+            break;
+          }
+        }
       } else {
-        data_dup = o_strndup(data, size); // Force value to end with a NULL character
-        if (con_info->max_post_param_size > 0) {
-          if (off > con_info->max_post_param_size) {
-            return MHD_YES;
-          } else if (off + size > con_info->max_post_param_size) {
-            cur_size = con_info->max_post_param_size - off;
+        cur_data = u_map_get((struct _u_map *)con_info->request->map_post_body, key);
+        cur_size = u_map_get_length((struct _u_map *)con_info->request->map_post_body, key);
+        if (cur_data != NULL) {
+          if (off) {
+            data_concat = msprintf("%s%s", cur_data, data);
+          } else {
+            data_concat = msprintf("%s,%s", cur_data, data);
+          }
+          if (u_map_put((struct _u_map *)con_info->request->map_post_body, key, data_concat) != U_OK) {
+            ret = MHD_NO;
+            break;
+          }
+        } else {
+          if (u_map_put((struct _u_map *)con_info->request->map_post_body, key, data) != U_OK) {
+            ret = MHD_NO;
+            break;
           }
         }
       }
-    } else {
-      return MHD_NO;
-    }
-
-    if (filename != NULL) {
-      filename_param = msprintf("%s_filename", key);
-      if (!u_map_has_key((struct _u_map *)con_info->request->map_post_body, filename_param) && u_map_put((struct _u_map *)con_info->request->map_post_body, filename_param, filename) != U_OK) {
-        y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error u_map_put filename value");
-      }
-      o_free(filename_param);
-    }
-
-    if (cur_size > 0 && data_dup != NULL && u_map_put_binary((struct _u_map *)con_info->request->map_post_body, key, data_dup, off, cur_size + 1) == U_OK) {
-      o_free(data_dup);
-      return MHD_YES;
-    } else {
-      o_free(data_dup);
-      return MHD_NO;
-    }
+      
+    } while (0);
+    o_free(data_concat);
+    o_free(filename_param);
   }
+  return ret;
 }
 
 #if MHD_VERSION >= 0x00096100
@@ -392,7 +435,6 @@ static int ulfius_webservice_dispatcher (void * cls,
                                          size_t * upload_data_size,
                                          void ** con_cls) {
 #endif
-
   struct _u_endpoint * endpoint_list = ((struct _u_instance *)cls)->endpoint_list, ** current_endpoint_list = NULL, * current_endpoint = NULL;
   struct connection_info_struct * con_info = * con_cls;
   int mhd_ret = MHD_NO, callback_ret = U_OK, i, close_loop = 0, inner_error = U_OK, mhd_response_flag;
@@ -479,7 +521,8 @@ static int ulfius_webservice_dispatcher (void * cls,
     content_type = (char*)u_map_get_case(con_info->request->map_header, ULFIUS_HTTP_HEADER_CONTENT);
 
     // Set POST Processor if content-type is properly set
-    if (content_type != NULL && (0 == o_strncmp(MHD_HTTP_POST_ENCODING_FORM_URLENCODED, content_type, o_strlen(MHD_HTTP_POST_ENCODING_FORM_URLENCODED)) ||
+    if (content_type != NULL && 
+       (0 == o_strncmp(MHD_HTTP_POST_ENCODING_FORM_URLENCODED, content_type, o_strlen(MHD_HTTP_POST_ENCODING_FORM_URLENCODED)) ||
         0 == o_strncmp(MHD_HTTP_POST_ENCODING_MULTIPART_FORMDATA, content_type, o_strlen(MHD_HTTP_POST_ENCODING_MULTIPART_FORMDATA)))) {
       con_info->has_post_processor = 1;
       con_info->post_processor = MHD_create_post_processor (connection, ULFIUS_POSTBUFFERSIZE, mhd_iterate_post_data, (void *) con_info);
@@ -738,9 +781,9 @@ static int ulfius_webservice_dispatcher (void * cls,
                                            o_strcasestr(u_map_get_case(con_info->request->map_header, "upgrade"), U_WEBSOCKET_UPGRADE_VALUE)==NULL?"No Upgrade websocket header\n":"",
                                            o_strcasestr(u_map_get_case(con_info->request->map_header, "Connection"), "Upgrade")==NULL?"No Connection Upgrade header\n":"",
                                            u_map_get_case(con_info->request->map_header, "Sec-WebSocket-Key")==NULL?"No Sec-WebSocket-Key header\n":"",
-                                           o_strcmp(con_info->request->http_protocol, "HTTP/1.1")!=0?"Wrong HTTP Protocol":"",
+                                           o_strcmp(con_info->request->http_protocol, "HTTP/1.1")!=0?"Wrong HTTP Protocol\n":"",
                                            o_strcmp(u_map_get_case(con_info->request->map_header, "Sec-WebSocket-Version"), "13")!=0?"Wrong websocket version\n":"",
-                                           o_strcmp(con_info->request->http_verb, "GET")!=0?"Method is not GET":"");
+                                           o_strcmp(con_info->request->http_verb, "GET")!=0?"Method is not GET\n":"");
                 response->status = MHD_HTTP_BAD_REQUEST;
                 y_log_message(Y_LOG_LEVEL_DEBUG, "Ulfius - Error websocket connection: %s", response_buffer);
                 mhd_response = MHD_CREATE_RESPONSE_FROM_BUFFER_PIMPED (o_strlen(response_buffer), response_buffer, mhd_response_flag );
@@ -857,7 +900,7 @@ static int ulfius_webservice_dispatcher (void * cls,
           }
         }
 
-        if (!con_info->request->callback_position && ((struct _u_instance *)cls)->default_endpoint != NULL && ((struct _u_instance *)cls)->default_endpoint->callback_function != NULL) {
+        if (!con_info->request->callback_position && ((struct _u_instance *)cls)->default_endpoint != NULL && ((struct _u_instance *)cls)->default_endpoint->callback_function != NULL && mhd_response == NULL) {
           callback_ret = ((struct _u_instance *)cls)->default_endpoint->callback_function(con_info->request, response, ((struct _u_instance *)cls)->default_endpoint->user_data);
           // Test callback_ret to know what to do
           switch (callback_ret) {
@@ -957,6 +1000,9 @@ static int ulfius_webservice_dispatcher (void * cls,
           }
           MHD_destroy_response (mhd_response);
           // Free Response parameters
+          if (response->free_shared_data != NULL && response->shared_data != NULL) {
+            response->free_shared_data(response->shared_data);
+          }
           ulfius_clean_response_full(response);
           response = NULL;
         }
@@ -1469,7 +1515,7 @@ int ulfius_remove_endpoint(struct _u_instance * u_instance, const struct _u_endp
   return ret;
 }
 
-const struct _u_endpoint * ulfius_empty_endpoint() {
+const struct _u_endpoint * ulfius_empty_endpoint(void) {
   static struct _u_endpoint empty_endpoint;
 
   empty_endpoint.http_method = NULL;
@@ -1856,7 +1902,7 @@ char * ulfius_url_decode(const char * str) {
   }
 }
 
-int ulfius_global_init() {
+int ulfius_global_init(void) {
   int ret = U_OK;
   o_malloc_t malloc_fn;
   o_realloc_t realloc_fn;
@@ -1880,7 +1926,7 @@ int ulfius_global_init() {
   return ret;
 }
 
-void ulfius_global_close() {
+void ulfius_global_close(void) {
 #ifndef U_DISABLE_CURL
   curl_global_cleanup();
 #endif
